// Game.cpp: implementation of the Game class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "test.h"
#include "Game.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


///////////////////////////////////////////////////////////////////////
// Processus d'evolution du jeu
///////////////////////////////////////////////////////////////////////
UINT ThreadGame(LPVOID pParam)
{
   Game * mygame = (Game *)(pParam);

   while (!mygame->getGameEnd()) {
	   mygame->actionGame();
	   mygame->paintGame();
       Sleep(mygame->gameSleep);
   }
   return 0;
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
Game::Game(CWnd * _gameWin, Joueur * myPlayer, int mySpeed )
{

   // Extraction des informations provenant de l'objet graphique
   gameWin  = _gameWin;
   winDc    = gameWin->GetDC();	
   CRect   rtab;
   gameWin->GetClientRect(&rtab);
   // Creation du BitMap interne
   winW = rtab.Width();
   winH = rtab.Height();
   gameBitmap.CreateCompatibleBitmap(winDc,winW,winH);
   gameDc = new CDC();
   gameDc->CreateCompatibleDC(winDc);
   gameDc->SelectObject(&gameBitmap);

   // Init des variables interne a l'objet
   gameEnd    = false;
   gameWinner = false;
   numBall    = 0;
   stillBall  = 0;
   gameSleep  = mySpeed;
   gamePlayer = myPlayer;

   // Creation des objets metier
   gameTab    = new Tableau(rtab, RGB(0,0,0));

   // Lancement du thread associ au jeu
   gameThread = AfxBeginThread(ThreadGame,this,THREAD_PRIORITY_TIME_CRITICAL); 
}

Game::~Game()
{
    // Il vaut mieux que le thread soit mort !
	gameEnd = true;
	Sleep(gameSleep*4);				

	delete gameTab;
	for ( int i = 0 ; i < numBall   ; i++ ) delete gameBall[i];
}



////////////////////////////////////////////////////////////////
// Parametres
////////////////////////////////////////////////////////////////
bool Game::getGameEnd()
{
	return gameEnd;
}

void Game::setGameEnd()
{
	gameEnd = true;
}


////////////////////////////////////////////////////////////////
// Affichage
////////////////////////////////////////////////////////////////

void Game::paintGame()
{
    // Netoyage de l'ecran
    gameTab->afficheElement(gameDc);

    if ( !gameEnd ) {

	   // Affiche les elements du jeu
	   for ( int i = 0 ; i < numBall   ; i++ ) gameBall[i]->afficheElement(gameDc);

	} else {

	   // Affiche Game Over/Win
	   gameDc->SetTextColor(RGB(255,0,0));
	   gameDc->SetBkColor(RGB(0,0,0));
	   if ( !gameWinner ) {
          gameDc->TextOut(150,150,"Game Over !");
	   } else gameDc->TextOut(150,150,"Game Win !");
    }

	// Met a jour la zone graphique reelle
	winDc->BitBlt(0,0,winW,winH,gameDc,0,0,SRCCOPY);
    gameWin->UpdateWindow();
}


////////////////////////////////////////////////////////////////
// Lancement de la balle
////////////////////////////////////////////////////////////////
void Game::startBall()
{
	if ( gamePlayer->haveBall() ) {
       gameBall[numBall] = new Ball(new CRect(10,10,20,20 ),
		                            RGB(255,0,0));
       numBall++;
	   stillBall++;
	}
}

////////////////////////////////////////////////////////////////
// Evolution de la balle et du jeu
////////////////////////////////////////////////////////////////
void Game::actionGame()
{

	// pour chaque balle, on effectue son mouvement et on teste
	// les collisions avec les autres objets du jeu...
	for ( int i = 0 ; i < numBall ; i++ ) {
		   gameBall[i]->moveBall();

		   if ( gameTab->performCollision(gameBall[i]) == COL_LOST ) {
		      if ( --stillBall == 0 ) {
                 gameEnd     = true;
		         gameWinner  = false;
			  }
		   }
	}
}

