// Game.cpp: implementation of the Game class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "test.h"
#include "Game.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


///////////////////////////////////////////////////////////////////////
// Processus d'evolution du jeu
///////////////////////////////////////////////////////////////////////
UINT ThreadGame(LPVOID pParam)
{
   Game * mygame = (Game *)(pParam);

   while (!mygame->getGameEnd()) {
	   mygame->actionGame();
	   mygame->paintGame();
       Sleep(mygame->gameSleep);
   }
   return 0;
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
Game::Game(CWnd * _gameWin, Joueur * myPlayer, int mySpeed )
{

   // Extraction des informations provenant de l'objet graphique
   gameWin  = _gameWin;
   winDc    = gameWin->GetDC();	
   CRect   rtab;
   gameWin->GetClientRect(&rtab);
   // Creation du BitMap interne
   winW = rtab.Width();
   winH = rtab.Height();
   gameBitmap.CreateCompatibleBitmap(winDc,winW,winH);
   gameDc = new CDC();
   gameDc->CreateCompatibleDC(winDc);
   gameDc->SelectObject(&gameBitmap);

   // Init des variables interne a l'objet
   gameEnd    = false;
   gameWinner = false;
   numBall    = 0;
   stillBall  = 0;
   gameSleep  = mySpeed;
   gamePlayer = myPlayer;

   // Creation des objets metier
   gameTab    = new Tableau(rtab, RGB(0,0,0));
   gameBatte  = new Batte(CRect( (rtab.Width()/2)-20,
	                             (rtab.bottom -10),
								 (rtab.Width()/2)+20,
								 (rtab.bottom) ),
						  RGB(0,200,0),
                          gamePlayer );
   gameBatte->setMinMaxX(rtab.left,rtab.right);
   createLevel();

   // Lancement du thread associ au jeu
   gameThread = AfxBeginThread(ThreadGame,this,THREAD_PRIORITY_TIME_CRITICAL); 
}

Game::~Game()
{
    // Il vaut mieux que le thread soit mort !
	gameEnd = true;
	Sleep(gameSleep*4);				

	delete gameTab;
	delete gameBatte;
	for ( int i = 0 ; i < numBall   ; i++ ) delete gameBall[i];
}



////////////////////////////////////////////////////////////////
// Parametres
////////////////////////////////////////////////////////////////
bool Game::getGameEnd()
{
	return gameEnd;
}

void Game::setGameEnd()
{
	gameEnd = true;
}


////////////////////////////////////////////////////////////////
// Affichage
////////////////////////////////////////////////////////////////

void Game::paintGame()
{
    // Netoyage de l'ecran
    gameTab->afficheElement(gameDc);

    if ( !gameEnd ) {

	   // Affiche les elements du jeu
	   for ( int i = 0 ; i < numBall   ; i++ ) gameBall[i]->afficheElement(gameDc);
	   for (     i = 0 ; i < numBrique ; i++ ) gameBrique[i]->afficheElement(gameDc);
	   gameBatte->afficheElement(gameDc);

	} else {

	   // Affiche Game Over/Win
	   gameDc->SetTextColor(RGB(255,0,0));
	   gameDc->SetBkColor(RGB(0,0,0));
	   if ( !gameWinner ) {
          gameDc->TextOut(150,150,"Game Over !");
	   } else gameDc->TextOut(150,150,"Game Win !");
    }

	// Met a jour la zone graphique reelle
	winDc->BitBlt(0,0,winW,winH,gameDc,0,0,SRCCOPY);
    gameWin->UpdateWindow();
}


////////////////////////////////////////////////////////////////
// Lancement de la balle
////////////////////////////////////////////////////////////////
void Game::startBall()
{
	if ( gamePlayer->haveBall() ) {
       gameBall[numBall] = new Ball(new CRect( gameBatte->getRect().left 
		                                       + (gameBatte->getRect().Width()/2) -5,
											   gameBatte->getRect().top-10,
											   gameBatte->getRect().left 
		                                       + (gameBatte->getRect().Width()/2) +5,
											   gameBatte->getRect().top ),
		                            RGB(255,0,0));
       numBall++;
	   stillBall++;
	}
}

////////////////////////////////////////////////////////////////
// Evolution de la balle et du jeu
////////////////////////////////////////////////////////////////
void Game::actionGame()
{

	// pour chaque balle, on effectue son mouvement et on teste
	// les collisions avec les autres objets du jeu...
	for ( int i = 0 ; i < numBall ; i++ ) {
		   gameBall[i]->moveBall();

		   gameBatte->performCollision(gameBall[i]);
		   if ( gameTab->performCollision(gameBall[i]) == COL_LOST ) {
		      if ( --stillBall == 0 ) {
                 gameEnd     = true;
		         gameWinner  = false;
			  }
		   }

		   for ( int k = 0 ; k < numBrique ; k++ )
			   gameBrique[k]->performCollision(gameBall[i]);
	}

	// Teste la fin gagnante
	bool fin = true;
	for ( i = 0 ; i < numBrique ; i++ )
		if ( !gameBrique[i]->getStatus() ) fin = false;
	if ( fin ) {
		gameEnd = true;
		gameWinner = true;
	}
}

////////////////////////////////////////////////////////////////
// Deplacement de la batte
////////////////////////////////////////////////////////////////	
void Game::moveBatte(CPoint p)
{
	gameBatte->moveBatte(p.x);
}

////////////////////////////////////////////////////////////////
// Creation niveau
////////////////////////////////////////////////////////////////
void Game::createLevel()
{
   int    ran;
   CRect trect;
   CRect rtab;
   gameWin->GetClientRect(&rtab);

   numBrique = 0;
   for ( int y = 0 ; y < 5  ; y++) {
	   for ( int x = 0 ; x < 10 ; x++) {
          trect.SetRect((rtab.Width()/10)*x+1,
	                    (y+5)*10,
			  			(rtab.Width()/10)*(x+1)-1,
			  			(y+6)*10-1 );

          ran = rand() % 10;
          switch (ran) {
	  	    case 0:
		    case 1:
		    case 2:
		    case 3:
				  gameBrique[numBrique] = new BriqueDure(trect,gamePlayer);
				  break;
		    case 4:
		    case 5:
		    case 6:
		    case 7:
		    case 8:
		    case 9:
                  gameBrique[numBrique] = new Brique(trect,gamePlayer);
				  break;
                  
		  }
		  numBrique++;
	   }
   }
}